﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using BuzzGUI.Interfaces;
using BuzzGUI.Common;

namespace patternist
{
    /// <summary>
    /// Interaction logic for AssignmentDialog.xaml
    /// </summary>
    public partial class AssignmentDialog : Window, INotifyPropertyChanged
    {
        // Returned values:

        public int SelectedNote { get; private set; }
        public IMachine SelectedMachine { get; private set; }
        public IPattern SelectedPattern { get; private set; }
        public ISequence SelectedSequence { get; private set; }

        // Binding lists to fill the listboxes and combos:

        public static IEnumerable<string> BuzzNoteNames { get { return BuzzNote.Names; } }

        BindingList<string> validMachineNames = new BindingList<string>(
                Global.Buzz.Song.Sequences
                .Select(s => s.Machine)
                .Distinct()
                .Where(m => m.Patterns.Count > 0)
                .Select(m => m.Name).ToList());
        public BindingList<string> ValidMachineNames { get { return validMachineNames; } }

        BindingList<string> machinePatternNames = new BindingList<string>();
        public BindingList<string> MachinePatternNames { get { return machinePatternNames; } }

        BindingList<int> machineSequenceNumbers = new BindingList<int>();
        public BindingList<int> MachineSequenceNumbers { get { return machineSequenceNumbers; } }

        // Bindings for currently selected values:

        public string SelectedNoteName
        {
            get { return BuzzNote.ToString(SelectedNote); }
            set { SelectedNote = BuzzNote.Parse(value.ToString()); }
        }

        public string SelectedMachineName
        {
            get { if (SelectedMachine != null) return SelectedMachine.Name; else return null; }
            set
            {
                machinePatternNames.Clear();
                machineSequenceNumbers.Clear();
                SelectedMachine = Global.Buzz.Song.Machines.FirstOrDefault(m => m.Name == value);
                if (SelectedMachine != null)
                {
                    // populate pattern ListBox
                    machinePatternNames = new BindingList<string>(SelectedMachine.Patterns.Select(x => x.Name).ToList());

                    // populate sequence numbers ComboBox
                    var seqs = Global.Buzz.Song.Sequences.Where(s => s.Machine == SelectedMachine);
                    for (int i=0; i<seqs.Count(); i++)
                        machineSequenceNumbers.Add(i + 1);

                    SelectedPatternName = machinePatternNames[0];
                    SelectedSequenceNumber = machineSequenceNumbers[0];
                }

                // move these to setters?
                PropertyChanged.Raise(this, "MachinePatternNames");
                PropertyChanged.Raise(this, "MachineSequenceNumbers");
                PropertyChanged.Raise(this, "SelectedPatternName");
                PropertyChanged.Raise(this, "SelectedSequenceNumber");
                checkValid();
            }
        }

        public string SelectedPatternName
        {
            get { if (SelectedPattern == null) return null; else return SelectedPattern.Name; }
            set {
                if (SelectedMachine != null && value != null)
                    SelectedPattern = SelectedMachine.Patterns.FirstOrDefault(p => p.Name == value.ToString());
                checkValid();
            }
        }

        public int? SelectedSequenceNumber
        {
            get
            {
                if (SelectedSequence == null)
                    return null;
                var sequences = Global.Buzz.Song.Sequences.Where(s => s.Machine == SelectedMachine).ToList();
                var seqNum = sequences.IndexOf(SelectedSequence);
                if (seqNum == -1)
                    return null;
                else
                    return seqNum + 1;
            }
            set
            {
                if (value == null)
                    SelectedSequence = null;
                else
                {
                    if (SelectedMachine != null)
                        SelectedSequence = Global.Buzz.Song.Sequences.Where(s => s.Machine == SelectedMachine).ToList()[(int)value-1];
                }
                checkValid();
            }
        }



        public AssignmentDialog(Assignment assignment)
        {
            SelectedNoteName = "C-4";
            this.DataContext = this;
            InitializeComponent();
            if (assignment != null)
            {
                SelectedNoteName = assignment.SourceNoteName;
                SelectedMachineName = assignment.TargetMacName;
                SelectedPatternName = assignment.TargetPatName;
                SelectedSequenceNumber = assignment.TargetSeqNum;
                PropertyChanged.Raise(this, "SelectedNoteName");
                PropertyChanged.Raise(this, "SelectedMachineName");
                PropertyChanged.Raise(this, "SelectedPatternName");
                PropertyChanged.Raise(this, "SelectedSequenceNumber");
            }
            checkValid();
        }

        private void okButton_Click(object sender, RoutedEventArgs e)
        {
            // Dialog box accepted
            this.DialogResult = true;
        }

        private void checkValid()
        {
            okButton.IsEnabled = (
                SelectedMachine != null &&
                SelectedPattern != null &&
                SelectedSequence != null
                );
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}
